@@ -1,7 +1,7 @@
 /*
  * This program attempts to recover deleted data structures in a registry hive.
  *
- * Copyright (C) 2008-2009 Timothy D. Morgan
+ * Copyright (C) 2008-2010 Timothy D. Morgan
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: reglookup-recover.c 152 2009-06-02 20:00:38Z tim $
+ * $Id: reglookup-recover.c 173 2010-03-08 03:39:09Z tim $
  */
 
 #include <stdio.h>
@@ -39,16 +39,16 @@ char* registry_file = NULL;
 #include "common.c"
 
 
-char* getQuotedData(int fd, uint32 offset, uint32 length)
+char* getQuotedData(int fd, uint32_t offset, uint32_t length)
 {
-  uint8* buf;
+  uint8_t* buf;
   char* quoted_buf;
-  uint32 len;
+  uint32_t len;
 
   if((lseek(fd, offset, SEEK_SET)) == -1)
     return NULL;
 
-  buf = (uint8*)malloc(length);
+  buf = (uint8_t*)malloc(length);
   if(buf == NULL)
     return NULL;
 
@@ -65,7 +65,7 @@ char* getQuotedData(int fd, uint32 offset, uint32 length)
   return quoted_buf;
 }
 
-
+/* XXX: Somewhere in here, need to start looking for and handling classnames */
 void printKey(REGFI_FILE* f, REGFI_NK_REC* nk, const char* prefix)
 {
   char mtime[20];
@@ -74,11 +74,14 @@ void printKey(REGFI_FILE* f, REGFI_NK_REC* nk, const char* prefix)
   char* quoted_name = NULL;
   char* quoted_raw = "";
 
-  *tmp_time = nt_time_to_unix(&nk->mtime);
+  *tmp_time = regfi_nt2unix_time(&nk->mtime);
   tmp_time_s = gmtime(tmp_time);
   strftime(mtime, sizeof(mtime), "%Y-%m-%d %H:%M:%S", tmp_time_s);
 
-  quoted_name = quote_string(nk->keyname, key_special_chars);
+  /* XXX: Add command line option to choose output encoding */
+  regfi_interpret_keyname(f, nk, REGFI_ENCODING_ASCII, true);
+
+  quoted_name = get_quoted_keyname(nk);
   if (quoted_name == NULL)
   {
     quoted_name = malloc(1*sizeof(char));
@@ -103,34 +106,18 @@ void printKey(REGFI_FILE* f, REGFI_NK_REC* nk, const char* prefix)
 }
 
 
-void printValue(REGFI_FILE* f, const REGFI_VK_REC* vk, const char* prefix)
+void printValue(REGFI_FILE* f, REGFI_VK_REC* vk, const char* prefix)
 {
   char* quoted_value = NULL;
   char* quoted_name = NULL;
   char* quoted_raw = "";
   char* conv_error = NULL;
   const char* str_type = NULL;
-  uint32 size = vk->data_size;
 
-  /* Microsoft's documentation indicates that "available memory" is 
-   * the limit on value sizes.  Annoying.  We limit it to 1M which 
-   * should rarely be exceeded, unless the file is corrupt or 
-   * malicious. For more info, see:
-   *   http://msdn2.microsoft.com/en-us/library/ms724872.aspx
-   */
-  /* XXX: Should probably do something different here for this tool.
-   *      Also, It would be really nice if this message somehow included the
-   *      name of the current value we're having trouble with, since
-   *      stderr/stdout don't always sync nicely.
-   */
-  if(size > REGFI_VK_MAX_DATA_LENGTH)
-  {
-    fprintf(stderr, "WARN: value data size %d larger than "
-	    "%d, truncating...\n", size, REGFI_VK_MAX_DATA_LENGTH);
-    size = REGFI_VK_MAX_DATA_LENGTH;
-  }
+  /* XXX: Add command line option to choose output encoding */
+  regfi_interpret_valuename(f, vk, REGFI_ENCODING_ASCII, true);
   
-  quoted_name = quote_string(vk->valuename, key_special_chars);
+  quoted_name = get_quoted_valuename(vk);
   if (quoted_name == NULL)
   { /* Value names are NULL when we're looking at the "(default)" value.
      * Currently we just return a 0-length string to try an eliminate 
@@ -138,13 +125,21 @@ void printValue(REGFI_FILE* f, const REGFI_VK_REC* vk, const char* prefix)
      * in the output allows one to differentiate between the parent key and
      * this value.
      */
-    quoted_name = malloc(1*sizeof(char));
+    quoted_name = strdup("");
     if(quoted_name == NULL)
       bailOut(REGLOOKUP_EXIT_OSERR, "ERROR: Could not allocate sufficient memory.\n");
-    quoted_name[0] = '\0';
   }
 
-  quoted_value = data_to_ascii(vk->data, size, vk->type, &conv_error);
+  /* XXX: Add command line option to choose output encoding */
+  if(vk->data != NULL 
+     && !regfi_interpret_data(f, REGFI_ENCODING_ASCII, vk->type, vk->data))
+  {
+    fprintf(stderr, "WARN: Error occurred while interpreting data for VK record"
+	    " at offset 0x%.8X.\n", vk->offset);
+  }
+  printMsgs(f);
+
+  quoted_value = data_to_ascii(vk->data, &conv_error);
   if(quoted_value == NULL)
   {
     quoted_value = malloc(1*sizeof(char));
@@ -227,10 +222,10 @@ void printSK(REGFI_FILE* f, REGFI_SK_REC* sk)
 }
 
 
-int printCell(REGFI_FILE* f, uint32 offset)
+int printCell(REGFI_FILE* f, uint32_t offset)
 {
   char* quoted_buf;
-  uint32 cell_length;
+  uint32_t cell_length;
   bool unalloc;
 
   if(!regfi_parse_cell(f->fd, offset, NULL, 0, &cell_length, &unalloc))
@@ -257,43 +252,44 @@ int printCell(REGFI_FILE* f, uint32 offset)
 char* getParentPath(REGFI_FILE* f, REGFI_NK_REC* nk)
 {
   void_stack* path_stack = void_stack_new(REGFI_MAX_DEPTH);
-  const REGFI_HBIN* hbin;
   REGFI_NK_REC* cur_ancestor;
   char* ret_val;
-  uint32 virt_offset, i, stack_size, ret_val_size, ret_val_used;
-  uint32 max_length;
+  uint32_t virt_offset, i, stack_size, ret_val_size, ret_val_used, offset;
+  int32_t max_size;
   REGFI_BUFFER* path_element;
   
   /* The path_stack size limit should guarantee that we don't recurse forever. */
   virt_offset = nk->parent_off;
   ret_val_size = 1; /* NUL */
   while(virt_offset != REGFI_OFFSET_NONE)
-  {  
-    hbin = regfi_lookup_hbin(f, virt_offset);
-    if(hbin == NULL)
+  {
+    offset = virt_offset+REGFI_REGF_SIZE;
+    max_size = regfi_calc_maxsize(f, offset);
+    if(max_size < 0)
       virt_offset = REGFI_OFFSET_NONE;
     else
     {
-      max_length = hbin->block_size + hbin->file_off 
-	- (virt_offset+REGFI_REGF_SIZE);
-      cur_ancestor = regfi_parse_nk(f, virt_offset+REGFI_REGF_SIZE, 
-				    max_length, true);
+      cur_ancestor = regfi_parse_nk(f, offset, max_size, true);
       printMsgs(f);
 
       if(cur_ancestor == NULL)
 	virt_offset = REGFI_OFFSET_NONE;
       else
       {
-	if(cur_ancestor->key_type & REGFI_NK_FLAG_ROOT)
+	if(cur_ancestor->flags & REGFI_NK_FLAG_ROOT)
 	  virt_offset = REGFI_OFFSET_NONE;
 	else
 	  virt_offset = cur_ancestor->parent_off;
 	
 	path_element = talloc(path_stack, REGFI_BUFFER);
 	if(path_element != NULL)
-	  path_element->buf = (uint8*)quote_string(cur_ancestor->keyname, 
-						   key_special_chars);
+	{
+	  /* XXX: Add command line option to choose output encoding */
+	  regfi_interpret_keyname(f, cur_ancestor, REGFI_ENCODING_ASCII, true);
 	  
+	  path_element->buf = (uint8_t*)get_quoted_keyname(cur_ancestor);
+	}
+ 
 	if(path_element == NULL || path_element->buf == NULL 
 	   || !void_stack_push(path_stack, path_element))
 	{
@@ -356,9 +352,9 @@ static void usage(void)
 }
 
 
-bool removeRange(range_list* rl, uint32 offset, uint32 length)
+bool removeRange(range_list* rl, uint32_t offset, uint32_t length)
 {
-  int32 rm_idx;
+  int32_t rm_idx;
   const range_list_element* cur_elem;
 
   rm_idx = range_list_find(rl, offset);
@@ -418,7 +414,7 @@ int extractVKs(REGFI_FILE* f,
 {
   const range_list_element* cur_elem;
   REGFI_VK_REC* vk;
-  uint32 i, j;
+  uint32_t i, j;
 
   for(i=0; i < range_list_size(unalloc_cells); i++)
   {
@@ -455,76 +451,136 @@ int extractVKs(REGFI_FILE* f,
 }
 
 
-int extractDataCells(REGFI_FILE* f,
+int extractDataCells(REGFI_FILE* file,
 		     range_list* unalloc_cells,
 		     range_list* unalloc_values)
 {
   const range_list_element* cur_elem;
   REGFI_VK_REC* vk;
-  const REGFI_HBIN* hbin;
+  range_list* bd_cells;
   REGFI_BUFFER data;
-  uint32 i, off, data_offset, data_maxsize;
+  uint32_t i, j, offset, cell_length, length;
+  int32_t max_size;
+  bool unalloc;
 
+  bd_cells = range_list_new();
+  if(bd_cells == NULL)
+    return 10;
+
+  data.buf = NULL;
+  data.len = 0;
   for(i=0; i<range_list_size(unalloc_values); i++)
   {
     cur_elem = range_list_get(unalloc_values, i);
     vk = (REGFI_VK_REC*)cur_elem->data;
     if(vk == NULL)
-      return 40;
+      return 11;
 
-    if(vk->data_size == 0)
-      vk->data = NULL;
-    else
+    length = vk->data_size;
+    vk->data = NULL;
+    if(vk->data_size != 0)
     {
-      off = vk->data_off+REGFI_REGF_SIZE;
+      offset = vk->data_off+REGFI_REGF_SIZE;
 
       if(vk->data_in_offset)
+	data = regfi_parse_little_data(file, vk->data_off, 
+				       length, false);
+      else
       {
-	data = regfi_load_data(f, vk->type, vk->data_off,
-			       vk->data_size, 4,
-			       vk->data_in_offset, false);
-	vk->data = data.buf;
-	vk->data_size = data.len;
-      }
-      else if(range_list_has_range(unalloc_cells, off, vk->data_size))
-      {
-	hbin = regfi_lookup_hbin(f, vk->data_off);
-	if(hbin)
+	max_size = regfi_calc_maxsize(file, offset);
+	if(max_size >= 0 
+	   && regfi_parse_cell(file->fd, offset, NULL, 0,
+			       &cell_length, &unalloc)
+	   && (cell_length & 0x00000007) == 0
+	   && cell_length <= max_size)
 	{
-	  data_offset = vk->data_off+REGFI_REGF_SIZE;
-	  data_maxsize = hbin->block_size + hbin->file_off - data_offset;
-	  data = regfi_load_data(f, vk->type, data_offset, 
-				 vk->data_size, data_maxsize, 
-				 vk->data_in_offset, false);
-	  vk->data = data.buf;
-	  vk->data_size = data.len;
-
-	  if(vk->data != NULL)
+	  if(cell_length - 4 < length)
 	  {
-	    /* XXX: The following may not make sense now in light of big data
-	     *      records.
+	    /* Multi-cell "big data" */
+
+	    /* XXX: All big data records thus far have been 16 bytes long.  
+	     *      Should we check for this precise size instead of just 
+	     *      relying upon the above check?
 	     */
-	    /* XXX: This strict checking prevents partial recovery of data 
-	     *      cells.  Also, see code for regfi_load_data and note that
-	     *      lengths indicated in VK records are sometimes just plain 
-	     *      wrong.  Need a feedback mechanism to be more fuzzy with 
-	     *      data cell lengths and the ranges removed. 
-	     *
-	     *      The introduction of REGFI_BUFFER in regfi_load_data has 
-	     *      fixed some of this.  Should review again with respect to 
-	     *      the other issues mentioned above though.
-	     */
-	    /* A data record was recovered. Remove from unalloc_cells. */
-	    if(!removeRange(unalloc_cells, off, vk->data_size))
-	      return 50;
+	    if (file->major_version >= 1 && file->minor_version >= 5)
+	    {
+	      /* Attempt to parse a big data record */
+	      data = regfi_load_big_data(file, offset, length, 
+					 cell_length, bd_cells, false);
+
+	      /* XXX: if this turns out NULL, should fall back to truncating cell */
+	      if(data.buf != NULL)
+	      {
+		for(j=0; j<range_list_size(bd_cells); j++)
+		{
+		  cur_elem = range_list_get(bd_cells, j);
+		  if(cur_elem == NULL)
+		    return 20;
+		  if(!range_list_has_range(unalloc_cells,
+					   cur_elem->offset, 
+					   cur_elem->length))
+		  {
+		    fprintf(stderr, 
+			    "WARN: Successfully parsed big data at offset"
+			    " 0x%.8X was rejected because some substructure"
+			    " (offset=0x%.8X) is allocated or used in other"
+			    " recovered structures.\n",
+			    offset, cur_elem->offset);
+		    talloc_free(data.buf);
+		    data.buf = NULL;
+		    data.len = 0;
+		    break;
+		  }
+		}
+		
+		if(data.buf != NULL)
+		{
+		  for(j=0; j<range_list_size(bd_cells); j++)
+		  {
+		    cur_elem = range_list_get(bd_cells, j);
+		    if(cur_elem == NULL)
+		      return 21;
+		    
+		    if(!removeRange(unalloc_cells, 
+				    cur_elem->offset,
+				    cur_elem->length))
+		    { return 22; }
+		  }
+		}
+	      }
+
+	    }
+	    else
+	    {
+	      fprintf(stderr, 
+		      "WARN: Data length (0x%.8X)"
+		      " larger than remaining cell length (0x%.8X)"
+		      " while parsing data record at offset 0x%.8X."
+		      " Truncating...\n",
+		      length, cell_length - 4, offset);
+	       length = cell_length - 4;
+	    }
+	  }
+	  
+	  /* Typical 1-cell data */
+	  if(range_list_has_range(unalloc_cells, offset, length))
+	  {
+	    data = regfi_parse_data(file, offset, length, false);
+	    if(data.buf != NULL)
+	      if(!removeRange(unalloc_cells, offset, length))
+		return 30;
 	  }
 	}
-	else
-	  vk->data = NULL;
       }
+      /* XXX: Need to come up with a different way to link these so the
+       *      vk->data item can be removed from the structure.
+       */
+      vk->data = regfi_buffer_to_data(data);
+      talloc_steal(vk, vk->data);
     }
   }
 
+  range_list_free(bd_cells);
   return 0;
 }
 
@@ -536,7 +592,7 @@ int extractKeys(REGFI_FILE* f,
 {
   const range_list_element* cur_elem;
   REGFI_NK_REC* key;
-  uint32 i, j;
+  uint32_t i, j;
   int error_code = 0;
 
   for(i=0; i < range_list_size(unalloc_cells); i++)
@@ -589,9 +645,9 @@ int extractValueLists(REGFI_FILE* f,
 {
   REGFI_NK_REC* nk;
   REGFI_VK_REC* vk;
-  const REGFI_HBIN* hbin;
   const range_list_element* cur_elem;
-  uint32 i, j, num_keys, off, values_length, max_length;
+  uint32_t i, j, num_keys, off, values_length;
+  int32_t max_size;
 
   num_keys=range_list_size(unalloc_keys);
   for(i=0; i<num_keys; i++)
@@ -603,14 +659,12 @@ int extractValueLists(REGFI_FILE* f,
 
     if(nk->num_values && (nk->values_off!=REGFI_OFFSET_NONE))
     {
-      hbin = regfi_lookup_hbin(f, nk->values_off);
-      
-      if(hbin != NULL)
+      off = nk->values_off + REGFI_REGF_SIZE;
+      max_size = regfi_calc_maxsize(f, off);
+      if(max_size >= 0)
       {
-	off = nk->values_off + REGFI_REGF_SIZE;
-	max_length = hbin->block_size + hbin->file_off - off;
 	nk->values = regfi_load_valuelist(f, off, nk->num_values, 
-					  max_length, false);
+					  max_size, false);
 	if(nk->values != NULL && nk->values->elements != NULL)
 	{
 	  /* Number of elements in the value list may be shorter than advertised 
@@ -618,7 +672,7 @@ int extractValueLists(REGFI_FILE* f,
 	   * only throw out the whole value list if it bleeds into an already 
 	   * parsed structure.
 	   */
-	  values_length = (nk->values->num_values+1)*sizeof(uint32);
+	  values_length = (nk->values->num_values+1)*sizeof(uint32_t);
 	  if(values_length != (values_length & 0xFFFFFFF8))
 	    values_length = (values_length & 0xFFFFFFF8) + 8;
 
@@ -682,7 +736,7 @@ int extractSKs(REGFI_FILE* f,
 {
   const range_list_element* cur_elem;
   REGFI_SK_REC* sk;
-  uint32 i, j;
+  uint32_t i, j;
 
   for(i=0; i < range_list_size(unalloc_cells); i++)
   {
@@ -733,7 +787,7 @@ int main(int argc, char** argv)
   char* tmp_path;
   REGFI_NK_REC* tmp_key;
   REGFI_VK_REC* tmp_value;
-  uint32 argi, arge, i, j, ret, num_unalloc_keys;
+  uint32_t argi, arge, i, j, ret, num_unalloc_keys;
   
   /* Process command line arguments */
   if(argc < 2)
@@ -884,7 +938,10 @@ int main(int argc, char** argv)
     printKey(f, tmp_key, parent_paths[i]);
     if(tmp_key->num_values > 0 && tmp_key->values != NULL)
     {
-      tmp_name = quote_string(tmp_key->keyname, key_special_chars);
+      /* XXX: Add command line option to choose output encoding */
+      regfi_interpret_keyname(f, tmp_key, REGFI_ENCODING_ASCII, true);
+
+      tmp_name = get_quoted_keyname(tmp_key);
       tmp_path = (char*)malloc(strlen(parent_paths[i])+strlen(tmp_name)+2);
       if(tmp_path == NULL)
       {
