@@ -1,7 +1,8 @@
 /*
  * A utility to read a Windows NT and later registry files.
  *
- * Copyright (C) 2005-2009 Timothy D. Morgan
+ * Copyright (C) 2005-2010 Timothy D. Morgan
+ * Copyright (C) 2010 Tobias Mueller (portions of '-i' code)
  * Copyright (C) 2002 Richard Sharpe, rsharpe@richardsharpe.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
  *
- * $Id: reglookup.c 150 2009-03-02 02:17:46Z tim $
+ * $Id: reglookup.c 172 2010-03-08 03:04:34Z tim $
  */
 
 
@@ -30,6 +31,7 @@
 #include "void_stack.h"
 
 /* Globals, influenced by command line parameters */
+bool print_value_mtime = false;
 bool print_verbose = false;
 bool print_security = false;
 bool print_header = true;
@@ -49,28 +51,18 @@ REGFI_FILE* f;
 #include "common.c"
 
 
-void printValue(const REGFI_VK_REC* vk, char* prefix)
+void printValue(REGFI_ITERATOR* iter, const REGFI_VK_REC* vk, char* prefix)
 {
+  REGFI_DATA* data;
   char* quoted_value = NULL;
   char* quoted_name = NULL;
   char* conv_error = NULL;
   const char* str_type = NULL;
-  uint32 size = vk->data_size;
+  char mtime[20];
+  time_t tmp_time[1];
+  struct tm* tmp_time_s = NULL;
 
-  /* Microsoft's documentation indicates that "available memory" is 
-   * the limit on value sizes.  Annoying.  We limit it to 1M which 
-   * should rarely be exceeded, unless the file is corrupt or 
-   * malicious. For more info, see:
-   *   http://msdn2.microsoft.com/en-us/library/ms724872.aspx
-   */
-  if(size > REGFI_VK_MAX_DATA_LENGTH)
-  {
-    fprintf(stderr, "WARN: value data size %d larger than "
-	    "%d, truncating...\n", size, REGFI_VK_MAX_DATA_LENGTH);
-    size = REGFI_VK_MAX_DATA_LENGTH;
-  }
-  
-  quoted_name = quote_string(vk->valuename, key_special_chars);
+  quoted_name = get_quoted_valuename(vk);
   if (quoted_name == NULL)
   { /* Value names are NULL when we're looking at the "(default)" value.
      * Currently we just return a 0-length string to try an eliminate 
@@ -83,16 +75,13 @@ void printValue(const REGFI_VK_REC* vk, char* prefix)
       bailOut(REGLOOKUP_EXIT_OSERR, "ERROR: Could not allocate sufficient memory.\n");
     quoted_name[0] = '\0';
   }
+  
+  data = regfi_iterator_fetch_data(iter, vk);
 
-  if(vk->data == NULL)
+  printMsgs(iter->f);
+  if(data != NULL)
   {
-    if(print_verbose)
-      fprintf(stderr, "INFO: While quoting value for '%s/%s', "
-	      "data pointer was NULL.\n", prefix, quoted_name);
-  }
-  else
-  {
-    quoted_value = data_to_ascii(vk->data, size, vk->type, &conv_error);
+    quoted_value = data_to_ascii(data, &conv_error);
     if(quoted_value == NULL)
     {
       if(conv_error == NULL)
@@ -102,29 +91,39 @@ void printValue(const REGFI_VK_REC* vk, char* prefix)
 	fprintf(stderr, "WARN: Could not quote value for '%s/%s'.  "
 		"Returned error: %s\n", prefix, quoted_name, conv_error);
     }
-    else if(conv_error != NULL && print_verbose)
-      fprintf(stderr, "INFO: While quoting value for '%s/%s', "
+    else if(conv_error != NULL)
+      fprintf(stderr, "WARN: While quoting value for '%s/%s', "
 	      "warning returned: %s\n", prefix, quoted_name, conv_error);
+    regfi_free_data(data);
   }
 
+  if(print_value_mtime)
+  {
+    *tmp_time = regfi_nt2unix_time(&iter->cur_key->mtime);
+    tmp_time_s = gmtime(tmp_time);
+    strftime(mtime, sizeof(mtime), "%Y-%m-%d %H:%M:%S", tmp_time_s);
+  }
+  else
+    mtime[0] = '\0';
+
   str_type = regfi_type_val2str(vk->type);
   if(print_security)
   {
     if(str_type == NULL)
-      printf("%s/%s,0x%.8X,%s,,,,,\n", prefix, quoted_name,
-	     vk->type, quoted_value);
+      printf("%s/%s,0x%.8X,%s,%s,,,,\n", prefix, quoted_name,
+	     vk->type, quoted_value, mtime);
     else
-      printf("%s/%s,%s,%s,,,,,\n", prefix, quoted_name,
-	     str_type, quoted_value);
+      printf("%s/%s,%s,%s,%s,,,,\n", prefix, quoted_name,
+	     str_type, quoted_value, mtime);
   }
   else
   {
     if(str_type == NULL)
-      printf("%s/%s,0x%.8X,%s,\n", prefix, quoted_name,
-	     vk->type, quoted_value);
+      printf("%s/%s,0x%.8X,%s,%s\n", prefix, quoted_name,
+	     vk->type, quoted_value, mtime);
     else
-      printf("%s/%s,%s,%s,\n", prefix, quoted_name,
-	     str_type, quoted_value);
+      printf("%s/%s,%s,%s,%s\n", prefix, quoted_name,
+	     str_type, quoted_value, mtime);
   }
 
   if(quoted_value != NULL)
@@ -142,7 +141,7 @@ char** splitPath(const char* s)
   const char* cur = s;
   char* next = NULL;
   char* copy;
-  uint32 ret_cur = 0;
+  uint32_t ret_cur = 0;
 
   ret_val = (char**)malloc((REGFI_MAX_DEPTH+1+1)*sizeof(char**));
   if (ret_val == NULL)
@@ -189,7 +188,7 @@ char** splitPath(const char* s)
 
 void freePath(char** path)
 {
-  uint32 i;
+  uint32_t i;
 
   if(path == NULL)
     return;
@@ -205,14 +204,14 @@ void freePath(char** path)
 char* iter2Path(REGFI_ITERATOR* i)
 {
   const REGFI_ITER_POSITION* cur;
-  uint32 buf_left = 127;
-  uint32 buf_len = buf_left+1;
-  uint32 name_len = 0;
-  uint32 grow_amt;
+  const REGFI_NK_REC* tmp_key;
+  uint32_t buf_left = 127;
+  uint32_t buf_len = buf_left+1;
+  uint32_t name_len = 0;
+  uint32_t grow_amt;
   char* buf;
   char* new_buf;
   char* name;
-  const char* cur_name;
   void_stack_iterator* iter;
   
   buf = (char*)malloc((buf_len)*sizeof(char));
@@ -240,17 +239,18 @@ char* iter2Path(REGFI_ITERATOR* i)
   {
     cur = void_stack_iterator_next(iter);
     if (cur == NULL)
-      cur_name = i->cur_key->keyname;
+      tmp_key = i->cur_key;
     else
-      cur_name = cur->nk->keyname;
+      tmp_key = cur->nk;
+
+    name = get_quoted_keyname(tmp_key);
 
     buf[buf_len-buf_left-1] = '/';
     buf_left -= 1;
-    name = quote_string(cur_name, key_special_chars);
     name_len = strlen(name);
     if(name_len+1 > buf_left)
     {
-      grow_amt = (uint32)(buf_len/2);
+      grow_amt = (uint32_t)(buf_len/2);
       buf_len += name_len+1+grow_amt-buf_left;
       if((new_buf = realloc(buf, buf_len)) == NULL)
       {
@@ -280,7 +280,7 @@ void printValueList(REGFI_ITERATOR* iter, char* prefix)
   while(value != NULL)
   {
     if(!type_filter_enabled || (value->type == type_filter))
-      printValue(value, prefix);
+      printValue(iter, value, prefix);
     regfi_free_value(value);
     value = regfi_iterator_next_value(iter);
     printMsgs(iter->f);
@@ -296,14 +296,14 @@ void printKey(REGFI_ITERATOR* iter, char* full_path)
   char* sacl = NULL;
   char* dacl = NULL;
   char* quoted_classname;
-  char* error_msg = NULL;
   char mtime[20];
   time_t tmp_time[1];
   struct tm* tmp_time_s = NULL;
   const REGFI_SK_REC* sk;
   const REGFI_NK_REC* k = regfi_iterator_cur_key(iter);
+  REGFI_CLASSNAME* classname;
 
-  *tmp_time = nt_time_to_unix(&k->mtime);
+  *tmp_time = regfi_nt2unix_time(&k->mtime);
   tmp_time_s = gmtime(tmp_time);
   strftime(mtime, sizeof(mtime), "%Y-%m-%d %H:%M:%S", tmp_time_s);
 
@@ -322,32 +322,31 @@ void printKey(REGFI_ITERATOR* iter, char* full_path)
     if(dacl == NULL)
       dacl = empty_str;
 
-    if(k->classname != NULL)
+    classname = regfi_iterator_fetch_classname(iter, k);
+    printMsgs(iter->f);
+    if(classname != NULL)
     {
-      quoted_classname = quote_unicode((uint8*)k->classname, k->classname_length,
-				       key_special_chars, &error_msg);
+      if(classname->interpreted == NULL)
+      {
+	fprintf(stderr, "WARN: Could not convert class name"
+		" charset for key '%s'.  Quoting raw...\n", full_path);
+	quoted_classname = quote_buffer(classname->raw, classname->size,
+					key_special_chars);
+      }
+      else
+	quoted_classname = quote_string(classname->interpreted, 
+					key_special_chars);
+
       if(quoted_classname == NULL)
       {
-	if(error_msg == NULL)
-	  fprintf(stderr, "ERROR: Could not quote classname"
-		  " for key '%s' due to unknown error.\n", full_path);
-	else
-	{
-	  fprintf(stderr, "ERROR: Could not quote classname"
-		  " for key '%s' due to error: %s\n", full_path, error_msg);
-	  free(error_msg);
-	}
-      }
-      else if (error_msg != NULL)
-      {
-	if(print_verbose)
-	  fprintf(stderr, "INFO: While converting classname"
-		  " for key '%s': %s.\n", full_path, error_msg);
-	free(error_msg);
+	fprintf(stderr, "ERROR: Could not quote classname"
+		" for key '%s' due to unknown error.\n", full_path);
+	quoted_classname = empty_str;
       }
     }
     else
       quoted_classname = empty_str;
+    regfi_free_classname(classname);
 
     printMsgs(iter->f);
     printf("%s,KEY,,%s,%s,%s,%s,%s,%s\n", full_path, mtime, 
@@ -460,7 +459,7 @@ int retrievePath(REGFI_ITERATOR* iter, char** path)
   REGFI_VK_REC* value;
   char* tmp_path_joined;
   const char** tmp_path;
-  uint32 i;
+  uint32_t i;
   
   if(path == NULL)
     return -1;
@@ -510,7 +509,7 @@ int retrievePath(REGFI_ITERATOR* iter, char** path)
       bailOut(REGLOOKUP_EXIT_OSERR, "ERROR: Unexpected error before printValue.\n");
 
     if(!type_filter_enabled || (value->type == type_filter))
-      printValue(value, tmp_path_joined);
+      printValue(iter, value, tmp_path_joined);
 
     regfi_free_value(value);
     free(tmp_path);
@@ -554,6 +553,7 @@ static void usage(void)
   fprintf(stderr, "\t-S\t disables security descriptor output. (default)\n");
   fprintf(stderr, "\t-p\t restrict output to elements below this path.\n");
   fprintf(stderr, "\t-t\t restrict results to this specific data type.\n");
+  fprintf(stderr, "\t-i\t includes parent key modification times with child values.\n");
   fprintf(stderr, "\n");
 }
 
@@ -563,7 +563,7 @@ int main(int argc, char** argv)
   char** path = NULL;
   REGFI_ITERATOR* iter;
   int retr_path_ret;
-  uint32 argi, arge;
+  uint32_t argi, arge;
 
   /* Process command line arguments */
   if(argc < 2)
@@ -611,6 +611,8 @@ int main(int argc, char** argv)
       print_security = false;
     else if (strcmp("-v", argv[argi]) == 0)
       print_verbose = true;
+    else if (strcmp("-i", argv[argi]) == 0)
+      print_value_mtime = true;
     else
     {
       usage();
@@ -631,9 +633,13 @@ int main(int argc, char** argv)
   if(print_verbose)
     regfi_set_message_mask(f, REGFI_MSG_INFO|REGFI_MSG_WARN|REGFI_MSG_ERROR);
 
-  iter = regfi_iterator_new(f);
+  /* XXX: add command line option to choose output encoding */
+  iter = regfi_iterator_new(f, REGFI_ENCODING_ASCII);
   if(iter == NULL)
+  {
+    printMsgs(f);
     bailOut(REGLOOKUP_EXIT_OSERR, "ERROR: Couldn't create registry iterator.\n");
+  }
 
   if(print_header)
   {
